@@ -29,19 +29,24 @@ this is desired in the CONFIG_PRESET dictionary right after the start of the
 program and before any DHParser-function is invoked.
 """
 
+import os
+import sys
 import threading
-from typing import Dict, Any, Optional
+from typing import Dict, Any
 
 __all__ = ('ALLOWED_PRESET_VALUES',
            'validate_value',
            'access_presets',
            'finalize_presets',
            'get_preset_value',
+           'get_preset_values',
            'set_preset_value',
+           'read_local_config',
            'NO_DEFAULT',
            'THREAD_LOCALS',
            'access_thread_locals',
            'get_config_value',
+           'get_config_values',
            'set_config_value',
            'NEVER_MATCH_PATTERN')
 
@@ -178,6 +183,67 @@ def set_preset_value(key: str, value: Any, allow_new_key: bool=False):
     PRESETS_CHANGED = True
 
 
+def read_local_config(ini_filename: str) -> str:
+    """Reads a local config file and updates the presets
+    accordingly. If the file is not found at the given path,
+    the same base name will be tried in the current working
+    directory, then in the applications config-directory and,
+    ultimately, in the calling script's directory.
+    This configuration file must be in the .ini-file format
+    so that it can be parsed with "configparser" from the
+    Python standard library. Any key,value-pair under the
+    section "DHParser" will directly be transferred to the
+    configuration presets. For other sections, the section
+    name will added as a qualifier to the key:
+    "section.key". Thus only values under the "DHParser"
+    section modify the DHParser-configuration while
+    configuration parameters unter other sections are
+    free to be evaluated by the calling script and cannot
+    interfere with DHParser's configuration.
+
+    :param ini_filename: the file path and name of the configuration
+        file.
+    :returns:  the file path of the actually read .ini-file
+        or the empty string if no .ini-file with the given
+        name could be found either at the given path, in
+        the current working directory or in the calling
+        script's path.
+    """
+    import configparser
+    config = configparser.RawConfigParser()
+    config.optionxform = lambda option: option
+    if not os.path.exists(ini_filename):
+        # try cfg-file in current working directory next
+        basename = os.path.basename(ini_filename)
+        ini_filename = basename
+    if not os.path.exists(ini_filename):
+        # try cfg-file in the applications' config-directory
+        # TODO: use a more portable method
+        dirname = os.path.splitext(basename)[0]
+        cfg_filename = os.path.join(os.path.expanduser('~'), '.config', dirname, 'config.ini')
+        if os.path.exists(cfg_filename):
+            ini_filename = cfg_filename
+        else:
+            ini_filename = os.path.join(os.path.expanduser('~'), '.config', dirname, basename)
+    if not os.path.exists(ini_filename):
+        # try cfg-file in script-directory next
+        script_path = os.path.abspath(sys.modules['__main__'].__file__)
+        ini_filename = os.path.join(os.path.dirname(script_path), basename)
+    if os.path.exists(ini_filename) and config.read(ini_filename):
+        access_presets()
+        for section in config.sections():
+            if section.lower() == "dhparser":
+                for variable, value in config[section].items():
+                    set_preset_value(f"{variable}", eval(value))
+            else:
+                for variable, value in config[section].items():
+                    set_preset_value(f"{section}.{variable}", eval(value),
+                                     allow_new_key=True)
+        finalize_presets()
+        return ini_filename
+    return ''
+
+
 class NoDefault:
     pass
 NO_DEFAULT = NoDefault()
@@ -193,6 +259,17 @@ def get_preset_value(key: str, default: Any = NO_DEFAULT):
     return CONFIG_PRESET.get(key, default)
 
 
+def get_preset_values(key_pattern: str) -> Dict:
+    """Returns a dictionary of all presets that match `key_pattern`."""
+    global CONFIG_PRESET, ACCESSING_PRESETS
+    if not ACCESSING_PRESETS:
+        raise AssertionError('Presets must be made accessible with access_presets() first, '
+                             'before they can be read!')
+    import fnmatch
+    return {key: value for key, value in CONFIG_PRESET.items()
+            if fnmatch.fnmatchcase(key, key_pattern)}
+
+
 def access_thread_locals() -> Any:
     """Intitializes (if not done yet) and returns the thread local variable
     store. (Call this function before using THREAD_LOCALS.
@@ -205,6 +282,16 @@ def access_thread_locals() -> Any:
     return THREAD_LOCALS
 
 
+def _config_dict():
+    THREAD_LOCALS = access_thread_locals()
+    try:
+        cfg = THREAD_LOCALS.config
+    except AttributeError:
+        THREAD_LOCALS.config = dict()
+        cfg = THREAD_LOCALS.config
+    return cfg
+
+
 def get_config_value(key: str, default: Any = NO_DEFAULT) -> Any:
     """
     Retrieves a configuration value thread-safely.
@@ -214,23 +301,34 @@ def get_config_value(key: str, default: Any = NO_DEFAULT) -> Any:
     :return:     the value
     """
     with access_lock:
-        THREAD_LOCALS = access_thread_locals()
-        try:
-            cfg = THREAD_LOCALS.config
-        except AttributeError:
-            THREAD_LOCALS.config = dict()
-            cfg = THREAD_LOCALS.config
+        cfg = _config_dict()
         try:
             return cfg[key]
         except KeyError:
             access_presets()
             value = get_preset_value(key, default)
             finalize_presets()
-            THREAD_LOCALS.config[key] = value
+            cfg[key] = value
             return value
 
 
-def set_config_value(key: str, value: Any):
+def get_config_values(key_pattern: str) -> Dict:
+    """Returns a dictionary of all configuration entries that match
+    `key_pattern`."""
+    access_presets()
+    presets = get_preset_values(key_pattern)
+    finalize_presets()
+    import fnmatch
+    with access_lock:
+        cfg = _config_dict()
+        cfg_values = {key: value for key, value in cfg.items()
+                      if fnmatch.fnmatchcase(key, key_pattern)}
+        presets.update(cfg_values)
+        cfg.update(presets)
+    return presets
+
+
+def set_config_value(key: str, value: Any, allow_new_key: bool = False):
     """
     Changes a configuration value thread-safely. The configuration
     value will be set only for the current thread. In order to
@@ -240,12 +338,11 @@ def set_config_value(key: str, value: Any):
     :param value:  the value
     """
     with access_lock:
-        THREAD_LOCALS = access_thread_locals()
-        try:
-            cfg = THREAD_LOCALS.config
-        except AttributeError:
-            THREAD_LOCALS.config = dict()
-            cfg = THREAD_LOCALS.config
+        cfg = _config_dict()
+        if not allow_new_key and key not in cfg and key not in CONFIG_PRESET:
+            raise ValueError(
+                '"%s" is not a valid config variable. Use "allow_new_key=True" to '
+                'add new variables or choose one of %s' % (key, list(cfg.keys())))
         validate_value(key, value)
         cfg[key] = value
 
@@ -362,7 +459,7 @@ ALLOWED_PRESET_VALUES['xml_attribute_error_handling'] = frozenset({'ignore', 'fi
 # The concrete syntax tree, the abstract syntax tree or both.
 # Possible values are {'ast'}, {'cst'} or {'ast', 'cst'}
 # Default value: empty set
-CONFIG_PRESET['log_syntax_trees'] = set()
+CONFIG_PRESET['log_syntax_trees'] = frozenset()
 
 
 ########################################################################
