@@ -24,21 +24,28 @@ EXPERIMENTAL!!!
 
 
 import bisect
-import copy
 from enum import Enum, IntEnum
 import functools
 import sys
-from typing import Union, List, Tuple, Optional, Dict, Any, Generic, TypeVar, \
-    Iterator, Iterable, Callable, cast, get_type_hints
-
+from typing import Union, List, Tuple, Optional, Dict, Any, \
+    Iterator, Iterable, Callable, get_type_hints
+from DHParser.externallibs.typing_extensions import Generic, GenericMeta, \
+    ClassVar, Final, Protocol, NoReturn, TypeVar, Literal
 try:
-    from typing import ForwardRef
+    from typing import ForwardRef, _GenericAlias, _SpecialForm
 except ImportError:
     from typing import _ForwardRef  # Python 3.6 compatibility
     ForwardRef = _ForwardRef
-
-from DHParser.server import RPC_Table
-from DHParser.toolkit import JSON_Type, JSON_Dict
+    _GenericAlias = GenericMeta
+    _SpecialForm = Any
+try:
+   from DHParser.externallibs.typing_extensions import get_origin
+except ImportError:
+    def get_origin(typ):
+        try:
+            return typ.__origin__
+        except AttributeError:
+            return Generic
 
 
 #######################################################################
@@ -82,7 +89,7 @@ def gen_lsp_name(func_name: str, prefix: str = 'lsp_') -> str:
 
 
 def gen_lsp_table(lsp_funcs_or_instance: Union[Iterable[Callable], Any],
-                  prefix: str = 'lsp_') -> RPC_Table:
+                  prefix: str = 'lsp_') -> Dict[str, Callable]:
     """Creates an RPC from a list of functions or from the methods
     of a class that implement the language server protocol.
     The dictionary keys are derived from the function name by replacing an
@@ -138,119 +145,126 @@ except ImportError:
 #
 #######################################################################
 
+# The following functions have been copied from the Python
+# standard libraries typing-module. They have been adapted
+# to support a more flexible version of TypedDict
+# see also: <https://www.python.org/dev/peps/pep-0655/>
 
-if sys.version_info >= (3, 8):
-
-    from typing import _GenericAlias, _SpecialForm
-    from DHParser.externallibs.typing_extensions import Generic, ClassVar, \
-        Final, Protocol, NoReturn, TypeVar, TypedDict, PEP_560, get_origin
+def _type_convert(arg, module=None):
+    """For converting None to type(None), and strings to ForwardRef."""
+    if arg is None:
+        return type(None)
+    if isinstance(arg, str):
+        fwref = ForwardRef(arg)
+        if hasattr(fwref, '__forward_module__'):
+            fwref.__forward_module__ = module
+        return fwref
+    return arg
 
 
-    def _type_convert(arg, module=None):
-        """For converting None to type(None), and strings to ForwardRef."""
-        if arg is None:
-            return type(None)
-        if isinstance(arg, str):
-            fwref = ForwardRef(arg)
-            if hasattr(fwref, '__forward_module__'):
-                fwref.__forward_module__ = module
-            return fwref
+def _type_check(arg, msg, is_argument=True, module=None):
+    """Check that the argument is a type, and return it (internal helper).
+    As a special case, accept None and return type(None) instead. Also wrap strings
+    into ForwardRef instances. Consider several corner cases, for example plain
+    special forms like Union are not valid, while Union[int, str] is OK, etc.
+    The msg argument is a human-readable error message, e.g::
+        "Union[arg, ...]: arg should be a type."
+    We append the repr() of the actual value (truncated to 100 chars).
+    """
+    invalid_generic_forms = (Generic, Protocol)
+    if is_argument:
+        invalid_generic_forms = invalid_generic_forms + (ClassVar, Final)
+
+    arg = _type_convert(arg, module=module)
+    if (isinstance(arg, _GenericAlias) and
+            arg.__origin__ in invalid_generic_forms):
+        raise TypeError(f"{arg} is not valid as type argument")
+    if arg in (Any, NoReturn):
         return arg
+    if (sys.version_info >= (3, 7) and isinstance(arg, _SpecialForm)) \
+            or arg in (Generic, Protocol):
+        raise TypeError(f"Plain {arg} is not valid as type argument")
+    if isinstance(arg, (type, TypeVar, ForwardRef)):
+        return arg
+    if not callable(arg):
+        raise TypeError(f"{msg} Got {arg!r:.100}.")
+    return arg
 
 
-    def _type_check(arg, msg, is_argument=True, module=None):
-        """Check that the argument is a type, and return it (internal helper).
-        As a special case, accept None and return type(None) instead. Also wrap strings
-        into ForwardRef instances. Consider several corner cases, for example plain
-        special forms like Union are not valid, while Union[int, str] is OK, etc.
-        The msg argument is a human-readable error message, e.g::
-            "Union[arg, ...]: arg should be a type."
-        We append the repr() of the actual value (truncated to 100 chars).
+def _caller(depth=1, default='__main__'):
+    try:
+        return sys._getframe(depth + 1).f_globals.get('__name__', default)
+    except (AttributeError, ValueError):  # For platforms without _getframe()
+        return None
+
+
+def _new_typed_dict(meta, name, bases, ns) -> dict:
+    for base in bases:
+        if base is not dict and type(base) is not meta \
+                and get_origin(base) is not Generic:
+            raise TypeError('cannot inherit from both a TypedDict type '
+                            'and a non-TypedDict base class: '
+                            f'{base} does not have type {meta}; '
+                            f'origin: {get_origin(base)}')
+    tp_dict = type.__new__(meta, name, (dict,), ns)
+
+    annotations = {}
+    own_annotations = ns.get('__annotations__', {})
+    # own_annotation_keys = set(own_annotations.keys())
+    msg = "TypedDict('Name', {f0: t0, f1: t1, ...}); each t must be a type"
+    own_annotations = {
+        n: _type_check(tp, msg, module=tp_dict.__module__)
+        for n, tp in own_annotations.items()
+    }
+    required_keys = set()
+    optional_keys = set()
+
+    for base in bases:
+        annotations.update(base.__dict__.get('__annotations__', {}))
+        required_keys.update(base.__dict__.get('__required_keys__', ()))
+        optional_keys.update(base.__dict__.get('__optional_keys__', ()))
+
+    annotations.update(own_annotations)
+
+    total = True
+    for field, field_type in own_annotations.items():
+        if get_origin(field_type) is Union \
+                and type(None) in field_type.__args__:
+            optional_keys.add(field)
+            total = False
+        else:
+            required_keys.add(field)
+
+    tp_dict.__annotations__ = annotations
+    tp_dict.__required_keys__ = frozenset(required_keys)
+    tp_dict.__optional_keys__ = frozenset(optional_keys)
+    if not hasattr(tp_dict, '__total__'):
+        tp_dict.__total__ = total
+    return tp_dict
+
+
+class _TypedDictMeta(type):
+    def __new__(cls, name, bases, ns, total=True):
+        """Create new typed dict class object.
+        This method is called when TypedDict is subclassed,
+        or when TypedDict is instantiated. This way
+        TypedDict supports all three syntax forms described in its docstring.
+        Subclasses and instances of TypedDict return actual dictionaries.
         """
-        invalid_generic_forms = (Generic, Protocol)
-        if is_argument:
-            invalid_generic_forms = invalid_generic_forms + (ClassVar, Final)
+        return _new_typed_dict(_TypedDictMeta, name, bases, ns)
 
-        arg = _type_convert(arg, module=module)
-        if (isinstance(arg, _GenericAlias) and
-                arg.__origin__ in invalid_generic_forms):
-            raise TypeError(f"{arg} is not valid as type argument")
-        if arg in (Any, NoReturn):
-            return arg
-        if isinstance(arg, _SpecialForm) or arg in (Generic, Protocol):
-            raise TypeError(f"Plain {arg} is not valid as type argument")
-        if isinstance(arg, (type, TypeVar, ForwardRef)):
-            return arg
-        if not callable(arg):
-            raise TypeError(f"{msg} Got {arg!r:.100}.")
-        return arg
+    __call__ = dict  # static method
 
+    def __subclasscheck__(cls, other):
+        # Typed dicts are only for static structural subtyping.
+        if sys.version_info < (3, 7, 0):
+            return False # hack to support Python 3.6
+        raise TypeError('TypedDict does not support instance and class checks')
 
-
-    def _caller(depth=1, default='__main__'):
-        try:
-            return sys._getframe(depth + 1).f_globals.get('__name__', default)
-        except (AttributeError, ValueError):  # For platforms without _getframe()
-            return None
-
-
-    class _TypedDictMeta(type):
-        def __new__(cls, name, bases, ns, total=True):
-            """Create new typed dict class object.
-            This method is called when TypedDict is subclassed,
-            or when TypedDict is instantiated. This way
-            TypedDict supports all three syntax forms described in its docstring.
-            Subclasses and instances of TypedDict return actual dictionaries.
-            """
-            for base in bases:
-                if type(base) is not _TypedDictMeta and get_origin(base) is not Generic:
-                    raise TypeError('cannot inherit from both a TypedDict type '
-                                    'and a non-TypedDict base class')
-            tp_dict = type.__new__(_TypedDictMeta, name, (dict,), ns)
-
-            annotations = {}
-            own_annotations = ns.get('__annotations__', {})
-            own_annotation_keys = set(own_annotations.keys())
-            msg = "TypedDict('Name', {f0: t0, f1: t1, ...}); each t must be a type"
-            own_annotations = {
-                n: _type_check(tp, msg, module=tp_dict.__module__)
-                for n, tp in own_annotations.items()
-            }
-            required_keys = set()
-            optional_keys = set()
-
-            for base in bases:
-                annotations.update(base.__dict__.get('__annotations__', {}))
-                required_keys.update(base.__dict__.get('__required_keys__', ()))
-                optional_keys.update(base.__dict__.get('__optional_keys__', ()))
-
-            annotations.update(own_annotations)
-
-            total = True
-            for field, field_type in own_annotations.items():
-                if get_origin(field_type) is Union \
-                        and type(None) in field_type.__args__:
-                    optional_keys.add(field)
-                    total = False
-                else:
-                    required_keys.add(field)
-
-            tp_dict.__annotations__ = annotations
-            tp_dict.__required_keys__ = frozenset(required_keys)
-            tp_dict.__optional_keys__ = frozenset(optional_keys)
-            if not hasattr(tp_dict, '__total__'):
-                tp_dict.__total__ = total
-            return tp_dict
-
-        __call__ = dict  # static method
-
-        def __subclasscheck__(cls, other):
-            # Typed dicts are only for static structural subtyping.
-            raise TypeError('TypedDict does not support instance and class checks')
-
-        __instancecheck__ = __subclasscheck__
+    __instancecheck__ = __subclasscheck__
 
 
+if sys.version_info >= (3, 7) and not hasattr(sys, 'pypy_version_info'):
     def TypedDict(typename, fields=None, *, total=True, **kwargs):
         """A simple typed namespace. At runtime it is equivalent to a plain dict.
         TypedDict creates a dictionary type that expects all of its
@@ -297,152 +311,166 @@ if sys.version_info >= (3, 8):
 
         return _TypedDictMeta(typename, (), ns)
 
-    _TypedDict = type.__new__(_TypedDictMeta, 'TypedDict', (), {})
-    TypedDict.__mro_entries__ = lambda bases: (_TypedDict,)
+    GenericTypedDict = TypedDict
+
+else:  # Python Version 3.6
+    TypedDict = _TypedDictMeta('TypedDict', (dict,), {})
+    TypedDict.__module__ = __name__
+    class _GenericTypedDictMeta(GenericMeta):
+        def __new__(cls, name, bases, ns, total=True):
+            return _new_typed_dict(_GenericTypedDictMeta, name, bases, ns)
+        __call__ = dict
+        def __subclasscheck__(cls, other):
+            return False  # hack to support Python 3.6
+        __instancecheck__ = __subclasscheck__
+
+    GenericTypedDict = _GenericTypedDictMeta('TypedDict', (dict,), {})
+    GenericTypedDict.__module__ = __name__
+
+_TypedDict = type.__new__(_TypedDictMeta, 'TypedDict', (), {})
+TypedDict.__mro_entries__ = lambda bases: (_TypedDict,)
+
+# up to this point all functions have been copied and adapted from
+# the typing.py module of the Pyhton-STL
+
+def validate_type(val: Any, typ):
+    if isinstance(typ, _TypedDictMeta):
+        if not isinstance(val, Dict):
+            raise TypeError(f"{val} is not even a dictionary")
+        validate_TypedDict(val, typ)
+    elif hasattr(typ, '__args__'):
+        validate_compound_type(val, typ)
+    else:
+        if not isinstance(val, typ):
+            raise TypeError(f"{val} is not of type {typ}")
 
 
-    def validate_type(val: Any, typ):
-        if isinstance(typ, _TypedDictMeta):
+def validate_uniform_sequence(sequence: Iterable, typ):
+    if isinstance(typ, _TypedDictMeta):
+        for val in sequence:
             if not isinstance(val, Dict):
-                raise TypeError(f"{val} is not even a dictionary")
+                raise TypeError(f"{val} is not of type {typ}")
             validate_TypedDict(val, typ)
-        elif hasattr(typ, '__args__'):
+    elif hasattr(typ, '__args__'):
+        for val in sequence:
             validate_compound_type(val, typ)
-        else:
+    else:
+        for val in sequence:
             if not isinstance(val, typ):
                 raise TypeError(f"{val} is not of type {typ}")
 
 
-    def validate_uniform_sequence(sequence: Iterable, typ):
-        if isinstance(typ, _TypedDictMeta):
-            for val in sequence:
-                if not isinstance(val, Dict):
-                    raise TypeError(f"{val} is not of type {typ}")
-                validate_TypedDict(val, typ)
-        elif hasattr(typ, '__args__'):
-            for val in sequence:
-                validate_compound_type(val, typ)
-        else:
-            for val in sequence:
-                if not isinstance(val, typ):
-                    raise TypeError(f"{val} is not of type {typ}")
+def validate_compound_type(value: Any, T):
+    assert hasattr(T, '__args__')
+    if isinstance(value, get_origin(T)):
+        if isinstance(value, Dict):
+            assert len(T.__args__) == 2, str(T)
+            key_type, value_type = T.__args__
+            validate_uniform_sequence(value.keys(), key_type)
+            validate_uniform_sequence(value.values(), value_type)
+        elif isinstance(value, Tuple):
+            if len(T.__args__) == 2 and T.__args__[-1] is Ellipsis:
+                validate_uniform_sequence(value, T.__args__[0])
+            else:
+                if len(T.__args__) != len(value):
+                    raise TypeError(f"{value} is not of type {T}")
+                for item, typ in zip(value, T.__args__):
+                    validate_type(item, typ)
+        else:  # assume that value is of type List
+            if len(T.__args__) != 1:
+                raise ValueError(f"Unknown compound type {T}")
+            validate_uniform_sequence(value, T)
+    else:
+        raise TypeError(f"{value} is not of type {get_origin(T)}")
 
 
-    def validate_compound_type(value: Any, T):
-        assert hasattr(T, '__args__')
-        if isinstance(value, get_origin(T)):
+def validate_TypedDict(D: Dict, T: _TypedDictMeta):
+    assert isinstance(D, Dict), str(D)
+    assert isinstance(T, _TypedDictMeta), str(T)
+    type_errors = []
+    missing = T.__required_keys__ - D.keys()
+    if missing:
+        type_errors.append(f"Missing required keys: {missing}")
+    unexpected = D.keys() - (T.__required_keys__ | T.__optional_keys__)
+    if unexpected:
+        type_errors.append(f"Unexpected keys: {unexpected}")
+    for field, field_type in get_type_hints(T).items():
+        if field not in D:
+            continue
+        if isinstance(field_type, _TypedDictMeta):
+            value = D[field]
             if isinstance(value, Dict):
-                assert len(T.__args__) == 2, str(T)
-                key_type, value_type = T.__args__
-                validate_uniform_sequence(value.keys(), key_type)
-                validate_uniform_sequence(value.values(), value_type)
-            elif isinstance(value, Tuple):
-                if len(T.__args__) == 2 and T.__args__[-1] is Ellipsis:
-                    validate_uniform_sequence(value, T.__args__[0])
-                else:
-                    if len(T.__args__) != len(value):
-                        raise TypeError(f"{value} is not of type {T}")
-                    for item, typ in zip(value, T.__args__):
-                        validate_type(item, typ)
-            else:  # assume that value is of type List
-                if len(T.__args__) != 1:
-                    raise ValueError(f"Unknown compound type {T}")
-                validate_uniform_sequence(value, T)
-        else:
-            raise TypeError(f"{value} is not of type {get_origin(T)}")
-
-
-    def validate_TypedDict(D: Dict, T: _TypedDictMeta):
-        assert isinstance(D, Dict), str(D)
-        assert isinstance(T, _TypedDictMeta), str(T)
-        type_errors = []
-        missing = T.__required_keys__ - D.keys()
-        if missing:
-            type_errors.append(f"Missing required keys: {missing}")
-        unexpected = D.keys() - (T.__required_keys__ | T.__optional_keys__)
-        if unexpected:
-            type_errors.append(f"Unexpected keys: {unexpected}")
-        for field, field_type in get_type_hints(T).items():
-            if field not in D:
-                continue
-            if isinstance(field_type, _TypedDictMeta):
-                value = D[field]
-                if isinstance(value, Dict):
-                    validate_TypedDict(value, field_type)
-                else:
-                    type_errors.append(f"Field {field} is not a {field_type}")
-            elif get_origin(field_type) is Union:
-                value = D[field]
-                for union_typ in field_type.__args__:
-                    if isinstance(union_typ, _TypedDictMeta):
-                        if isinstance(value, Dict):
-                            try:
-                                validate_TypedDict(value, union_typ)
-                                break
-                            except TypeError:
-                                pass
-                    elif hasattr(union_typ, '__args__'):
+                validate_TypedDict(value, field_type)
+            else:
+                type_errors.append(f"Field {field} is not a {field_type}")
+        elif get_origin(field_type) is Union:
+            value = D[field]
+            for union_typ in field_type.__args__:
+                if isinstance(union_typ, _TypedDictMeta):
+                    if isinstance(value, Dict):
                         try:
-                            validate_compound_type(value, union_typ)
+                            validate_TypedDict(value, union_typ)
                             break
                         except TypeError:
                             pass
-                    elif isinstance(value, union_typ):
+                elif hasattr(union_typ, '__args__'):
+                    try:
+                        validate_compound_type(value, union_typ)
                         break
-                else:
-                    type_errors.append(f"Field {field} is not of {field_type}")
-            elif hasattr(field_type, '__args__'):
-                validate_compound_type(D[field], field_type)
-            elif isinstance(field_type, TypeVar):
-                pass  # for now
-            elif not isinstance(D[field], field_type):
-                type_errors.append(f"Field {field} is not a {field_type}")
-        if type_errors:
-            if len(type_errors) == 1:
-                raise TypeError(f"Type error in dictionary of type {T}: "
-                                + type_errors[0])
+                    except TypeError:
+                        pass
+                elif isinstance(value, union_typ):
+                    break
             else:
-                raise TypeError(f"Type errors in dictionary of type {T}:\n"
-                                + '\n'.join(type_errors))
+                type_errors.append(f"Field {field} is not of {field_type}")
+        elif hasattr(field_type, '__args__'):
+            validate_compound_type(D[field], field_type)
+        elif isinstance(field_type, TypeVar):
+            pass  # for now
+        elif not isinstance(D[field], field_type):
+            type_errors.append(f"Field {field} is not a {field_type}")
+    if type_errors:
+        if len(type_errors) == 1:
+            raise TypeError(f"Type error in dictionary of type {T}: "
+                            + type_errors[0])
+        else:
+            raise TypeError(f"Type errors in dictionary of type {T}:\n"
+                            + '\n'.join(type_errors))
 
 
-    def type_check(func: Callable) -> Callable:
-        arg_names = func.__code__.co_varnames[:func.__code__.co_argcount]
-        arg_types = get_type_hints(func)
-        return_type = arg_types.get('return', None)
-        if return_type is not None:  del arg_types['return']
-        assert arg_types or return_type, \
-            f'type_check-decorated "{func}" has no type annotations'
+def type_check(func: Callable) -> Callable:
+    arg_names = func.__code__.co_varnames[:func.__code__.co_argcount]
+    arg_types = get_type_hints(func)
+    return_type = arg_types.get('return', None)
+    if return_type is not None:  del arg_types['return']
+    assert arg_types or return_type, \
+        f'type_check-decorated "{func}" has no type annotations'
 
-        @functools.wraps(func)
-        def guard(*args, **kwargs):
-            nonlocal arg_names, arg_types, return_type
-            arg_dict = {**dict(zip(arg_names, args)), **kwargs}
-            for name, typ in arg_types.items():
-                try:
-                    validate_type(arg_dict[name], typ)
-                except TypeError as e:
-                    raise TypeError(
-                        f'Parameter "{name}" of function "{func.__name__}" failed '
-                        f'the type-check, because: {str(e)}')
-                except KeyError as e:
-                    raise TypeError(f'Missing parameter {str(e)} in call of '
-                                    f'"{func.__name__}"')
-            ret = func(*args, **kwargs)
-            if return_type:
-                try:
-                    validate_type(ret, return_type)
-                except TypeError as e:
-                    raise TypeError(
-                        f'Value returned by function "{func.__name__}" failed '
-                        f'the type-check, because: {str(e)}')
-            return ret
+    @functools.wraps(func)
+    def guard(*args, **kwargs):
+        nonlocal arg_names, arg_types, return_type
+        arg_dict = {**dict(zip(arg_names, args)), **kwargs}
+        for name, typ in arg_types.items():
+            try:
+                validate_type(arg_dict[name], typ)
+            except TypeError as e:
+                raise TypeError(
+                    f'Parameter "{name}" of function "{func.__name__}" failed '
+                    f'the type-check, because: {str(e)}')
+            except KeyError as e:
+                raise TypeError(f'Missing parameter {str(e)} in call of '
+                                f'"{func.__name__}"')
+        ret = func(*args, **kwargs)
+        if return_type:
+            try:
+                validate_type(ret, return_type)
+            except TypeError as e:
+                raise TypeError(
+                    f'Value returned by function "{func.__name__}" failed '
+                    f'the type-check, because: {str(e)}')
+        return ret
 
-        return guard
-
-else:
-    def type_check(func: Callable) -> Callable:
-        return func
+    return guard
 
 
 #######################################################################
@@ -455,8 +483,13 @@ else:
 ##### BEGIN OF LSP SPECS
 
 
+
 integer = float
+
+
 uinteger = float
+
+
 decimal = float
 
 
@@ -467,19 +500,19 @@ class Message(TypedDict, total=True):
 class RequestMessage(Message, TypedDict, total=False):
     id: Union[int, str]
     method: str
-    params: Union[List, object, None]
+    params: Union[List, Dict, None]
 
 
 class ResponseMessage(Message, TypedDict, total=False):
     id: Union[int, str, None]
-    result: Union[str, float, bool, object, None]
+    result: Union[str, float, bool, Dict, None]
     error: Optional['ResponseError']
 
 
 class ResponseError(TypedDict, total=False):
     code: int
     message: str
-    data: Union[str, float, bool, List, object, None]
+    data: Union[str, float, bool, List, Dict, None]
 
 
 class ErrorCodes(IntEnum):
@@ -502,7 +535,7 @@ class ErrorCodes(IntEnum):
 
 class NotificationMessage(Message, TypedDict, total=False):
     method: str
-    params: Union[List, object, None]
+    params: Union[List, Dict, None]
 
 
 class CancelParams(TypedDict, total=True):
@@ -515,7 +548,7 @@ ProgressToken = Union[int, str]
 T = TypeVar('T')
 
 
-class ProgressParams(Generic[T]):
+class ProgressParams(Generic[T], GenericTypedDict, total=True):
     token: ProgressToken
     value: 'T'
 
@@ -624,7 +657,7 @@ class CreateFileOptions(TypedDict, total=False):
 
 
 class CreateFile(TypedDict, total=False):
-    kind: str
+    kind: 'create'
     uri: DocumentUri
     options: Optional[CreateFileOptions]
     annotationId: Optional[ChangeAnnotationIdentifier]
@@ -636,7 +669,7 @@ class RenameFileOptions(TypedDict, total=False):
 
 
 class RenameFile(TypedDict, total=False):
-    kind: str
+    kind: 'rename'
     oldUri: DocumentUri
     newUri: DocumentUri
     options: Optional[RenameFileOptions]
@@ -649,7 +682,7 @@ class DeleteFileOptions(TypedDict, total=False):
 
 
 class DeleteFile(TypedDict, total=False):
-    kind: str
+    kind: 'delete'
     uri: DocumentUri
     options: Optional[DeleteFileOptions]
     annotationId: Optional[ChangeAnnotationIdentifier]
@@ -741,7 +774,7 @@ class MarkdownClientCapabilities(TypedDict, total=False):
 
 
 class WorkDoneProgressBegin(TypedDict, total=False):
-    kind: str
+    kind: 'begin'
     title: str
     cancellable: Optional[bool]
     message: Optional[str]
@@ -749,14 +782,14 @@ class WorkDoneProgressBegin(TypedDict, total=False):
 
 
 class WorkDoneProgressReport(TypedDict, total=False):
-    kind: str
+    kind: 'report'
     cancellable: Optional[bool]
     message: Optional[str]
     percentage: Optional[int]
 
 
 class WorkDoneProgressEnd(TypedDict, total=False):
-    kind: str
+    kind: 'end'
     message: Optional[str]
 
 
@@ -772,7 +805,7 @@ class PartialResultParams(TypedDict, total=False):
     partialResultToken: Optional[ProgressToken]
 
 
-TraceValue = str
+TraceValue = Literal['off', 'messages', 'verbose']
 
 
 class InitializeParams(WorkDoneProgressParams, TypedDict, total=False):
@@ -2173,6 +2206,9 @@ class Moniker(TypedDict, total=False):
     identifier: str
     unique: UniquenessLevel
     kind: Optional[MonikerKind]
+
+
+
 ##### END OF LSP SPECS
 
 
