@@ -24,9 +24,9 @@ import sys
 from typing import cast
 
 scriptdir = os.path.dirname(os.path.abspath(__file__))
-i, k = scriptdir.find('DHParser-submodule'), len('DHParser-submodule')
-if i < 0:
-    i, k = scriptdir.find('DHParser'), len('DHParser')
+i = scriptdir.find('DHParser')
+k = len('DHParser-submodule') if scriptdir[i:].startswith('DHParser-submodule') \
+    else len('DHParser')
 if i >= 0:
     dhparserdir = scriptdir[:i + k]
     if dhparserdir not in sys.path:
@@ -34,31 +34,20 @@ if i >= 0:
 else:
     dhparserdir = ''
 
-templatedir = os.path.join(os.path.dirname(scriptdir.rstrip('/')), 'templates')
 
 from DHParser.compile import compile_source
-from DHParser.configuration import access_presets, finalize_presets, \
-    EBNF_ANY_SYNTAX_HEURISTICAL
-from DHParser.dsl import compileDSL, compile_on_disk
-from DHParser.error import is_error
+from DHParser.configuration import access_presets, set_preset_value, finalize_presets
+from DHParser.dsl import compileDSL, compile_on_disk, read_template
+from DHParser.error import is_error, has_errors, ERROR
 from DHParser.ebnf import get_ebnf_grammar, get_ebnf_transformer, get_ebnf_compiler
 from DHParser.log import start_logging
-from DHParser.toolkit import re
+from DHParser.toolkit import re, split_path
 
 
 LOGGING = False
 TEST_DIRNAME = 'tests_grammar'
 
 
-def read_template(template_name: str) -> str:
-    """
-    Reads a script-template from a template file named `template_name`
-    in the template-directory and returns it as a string.
-    """
-    with open(os.path.join(templatedir, template_name), 'r', encoding='utf-8') as f:
-        return f.read()
-
-
 def create_project(path: str):
     """Creates the a new DHParser-project in the given `path`.
     """
@@ -110,9 +99,9 @@ def create_project(path: str):
     create_file('README.md', README_TEMPLATE.format(name=name))
     reldhparserdir = os.path.relpath(dhparserdir, os.path.abspath('.'))
     create_file('tst_%s_grammar.py' % name, GRAMMAR_TEST_TEMPLATE.format(
-        name=name, reldhparserdir=reldhparserdir))
+        name=name, reldhparserdir=str(split_path(reldhparserdir))[1:-1]))
     create_file('%sServer.py' % name, SERVER_TEMPLATE.replace('DSL', name).replace(
-        'RELDHPARSERDIR', reldhparserdir))
+        'RELDHPARSERDIR', str(split_path(reldhparserdir))[1:-1]))
     create_file('example.dsl', 'Life is but a walking shadow\n')
     os.chmod('tst_%s_grammar.py' % name, 0o755)
     os.chmod('%sServer.py' % name, 0o755)
@@ -124,13 +113,17 @@ def create_project(path: str):
     os.chdir(curr_dir)
 
 
-def selftest() -> bool:
+def selftest(silent: bool=False) -> bool:
     """Run a simple self-test of DHParser.
     """
-    print("DHParser selftest...")
-    print("\nSTAGE I:  Trying to compile EBNF-Grammar:\n")
+    if not silent:
+        print("DHParser selftest...")
+        print("\nSTAGE I:  Trying to compile EBNF-Grammar:\n")
     builtin_ebnf_parser = get_ebnf_grammar()
     docstring = str(builtin_ebnf_parser.__doc__)  # type: str
+    i = docstring.find('::')
+    if i >= 0:
+        docstring = docstring[i + 2::]
     ebnf_src = docstring[docstring.find('@'):]
     ebnf_transformer = get_ebnf_transformer()
     ebnf_compiler = get_ebnf_compiler('EBNF')
@@ -139,17 +132,20 @@ def selftest() -> bool:
         builtin_ebnf_parser, ebnf_transformer, ebnf_compiler)
     generated_ebnf_parser = cast(str, result)
 
-    if errors:
-        print("Selftest FAILED :-(")
-        print("\n\n".join(str(err) for err in errors))
+    if has_errors(errors, ERROR):
+        if not silent:
+            print("Selftest FAILED :-(")
+            print("\n\n".join(str(err) for err in errors))
         return False
-    print(generated_ebnf_parser)
-    print("\n\nSTAGE 2: Selfhosting-test: "
-          "Trying to compile EBNF-Grammar with generated parser...\n")
+    if not silent:
+        print(generated_ebnf_parser)
+        print("\n\nSTAGE 2: Selfhosting-test: "
+              "Trying to compile EBNF-Grammar with generated parser...\n")
     selfhosted_ebnf_parser = compileDSL(ebnf_src, None, generated_ebnf_parser,
                                         ebnf_transformer, ebnf_compiler)
     # ebnf_compiler.gen_transformer_skeleton()
-    print(selfhosted_ebnf_parser)
+    if not silent:
+        print(selfhosted_ebnf_parser)
     return True
 
 
@@ -191,8 +187,8 @@ def main():
     """Creates a project (if a project name has been passed as command line
     parameter) or runs a quick self-test.
     """
-    config = access_presets()
-    config['syntax_variant'] = EBNF_ANY_SYNTAX_HEURISTICAL
+    access_presets()
+    set_preset_value('syntax_variant', 'heuristic')  # TODO: Bugs in Transformation-Table
     finalize_presets()
     if len(sys.argv) > 1:
         if sys.argv[1].lower() == "--selftest":
