@@ -27,11 +27,12 @@ import sys
 scriptpath = os.path.dirname(__file__) or '.'
 sys.path.append(os.path.abspath(os.path.join(scriptpath, '..')))
 
-from DHParser.configuration import get_config_value, set_config_value, INDENTED_SERIALIZATION, \
-    SXPRESSION_SERIALIZATION
+from DHParser.configuration import get_config_value, set_config_value
 from DHParser.syntaxtree import Node, RootNode, parse_sxpr, parse_xml, flatten_sxpr, \
-    flatten_xml, parse_json_syntaxtree, ZOMBIE_TAG, EMPTY_NODE, ALL_NODES, next_context, \
-    prev_context, serialize_context
+    flatten_xml, parse_json, ZOMBIE_TAG, EMPTY_NODE, ANY_NODE, next_context, \
+    prev_context, serialize_context, generate_context_mapping, map_pos_to_context, \
+    select_context_if, select_context, create_context_match_function, pick_context, \
+    LEAF_CONTEXT
 from DHParser.transform import traverse, reduce_single_child, \
     replace_by_single_child, flatten, remove_empty, remove_whitespace
 from DHParser.ebnf import get_ebnf_grammar, get_ebnf_transformer, get_ebnf_compiler
@@ -52,6 +53,11 @@ class TestParseSxpression:
                           "if argument is not a tree!"
         except ValueError:
             pass
+        try:
+            tree = parse_sxpr('(a (b c)))')
+            assert False, "parse_sxpr() should raise a ValueError for too many matching brackets."
+        except ValueError:
+            pass
 
     def test_parse_s_expression_w_attributes(self):
         s = '(A `(attr "1") (B "X"))'
@@ -66,6 +72,23 @@ class TestParseSxpression:
         assert str(tree) == "LIUTPR. leg. 21 ..."
         assert tree.attr['unterbedeutungstiefe'] == '0'
 
+    def test_parse_s_expression_malformed(self):
+        try:
+            s = parse_sxpr('(A (B 1) (C (D (E 2) (F 3)) (G 4) (H (I 5) (J 6)) (K 7)')
+            assert False, "ValueError exptected!"
+        except ValueError:
+            pass
+
+    def test_endlessloop_error(self):
+        tree = parse_sxpr(r'(LINEFEED "\\")')
+        assert tree
+
+    def test_flatten_sxpr(self):
+        tree = parse_sxpr('(a (b "  ") (d (e f) (h i)))')
+        sxpr = tree.as_sxpr()
+        flat = flatten_sxpr(sxpr)
+        assert flat == '(a (b "  ") (d (e "f") (h "i")))'
+
 
 class TestParseXML:
     def test_roundtrip(self):
@@ -83,7 +106,7 @@ class TestParseXML:
         assert flatten_xml(tree.as_xml()) == \
                '<a><ANONYMOUS_Text__>  </ANONYMOUS_Text__><b>beta</b>' \
                '<ANONYMOUS_Text__>  </ANONYMOUS_Text__></a>'
-        assert tree.as_xml(inline_tags={'a'}, omit_tags={':Text'}) == '<a>  <b>beta</b>  </a>'
+        assert tree.as_xml(inline_tags={'a'}, string_tags={':Text'}) == '<a>  <b>beta</b>  </a>'
         tree = parse_xml(' <a>\n  <b>beta</b>\n</a> ')
         assert tree.as_xml(inline_tags={'a'}) == '<a><b>beta</b></a>'
 
@@ -92,6 +115,41 @@ class TestParseXML:
         flat_xml = flatten_xml(tree.as_xml())
         assert flat_xml == '<alpha><beta>gamma</beta></alpha>', flat_xml
 
+    def test_endlessloop_error(self):
+        tree = parse_xml(r'<LINEFEED>\\</LINEFEED>')
+        assert tree
+
+    def test_PI_and_DTD(self):
+        """PIs <?...> and DTDs <!...> and the like should politely be overlooked."""
+        testdata = """<!DOCTYPE nonsense>
+            <?xpacket begin='' id='W5M0MpCehiHzreSzNTczkc9d'?> 
+            <?xpacket begin="r" id="Arnold-Mueller2017a"?> 
+            <x:xmpmeta xmlns:x="adobe:ns:meta/"> 
+            <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"> 
+            <rdf:Description xmlns:bibtex="http://jabref.sourceforge.net/bibteXMP/" 
+            bibtex:bibtexkey="Arnold-Mueller2017a" 
+            bibtex:journal="Informationspraxis" 
+            bibtex:title="Wie permanent sind Permalinks?" 
+            bibtex:type="Article" 
+            bibtex:doi="http://dx.doi.org/10.11588/ip.2016.2.33483" 
+            bibtex:year="2017" 
+            bibtex:volume="3" 
+            bibtex:issue="1" 
+            bibtex:url="http://www.eckhartarnold.de/papers/2016_Permalinks/Arnold_Mueller_2016_Permalinks.html"> 
+            <bibtex:author>Eckhart Arnold</bibtex:author> 
+            </rdf:Description> 
+            <!-- comment -->
+            </rdf:RDF> 
+            </x:xmpmeta> 
+            <?xpacket end="r"?> 
+            <?xpacket end='r'?>"""
+        tree = parse_xml(testdata)
+        assert tree.tag_name == 'x:xmpmeta'
+        author = tree.pick('bibtex:author')
+        assert author and author.content == "Eckhart Arnold"
+        description = tree.pick('rdf:Description')
+        assert description.has_attr('bibtex:title')
+
 
 class TestParseJSON:
     tree = parse_sxpr('(a (b ä) (d (e ö) (h über)))').with_pos(0)
@@ -115,7 +173,7 @@ class TestParseJSON:
         tree_copy = Node.from_json_obj(json.loads(s))
         assert tree_copy.equals(self.tree, ignore_attr_order = sys.version_info < (3, 6))
         s = self.tree.as_json(indent=None, ensure_ascii=False)
-        tree_copy = parse_json_syntaxtree(s)
+        tree_copy = parse_json(s)
         # print(s)
         # print(self.tree.as_sxpr())
         # print(tree_copy.as_sxpr())
@@ -127,7 +185,7 @@ class TestParseJSON:
         n.attr['id'] = '007'
         # json
         json = n.as_json()
-        tree = parse_json_syntaxtree(json)
+        tree = parse_json(json)
         # print()
 
         # XML
@@ -151,6 +209,22 @@ class TestParseJSON:
         assert tree.pos == 46
         assert not 'pos' in tree.attr
 
+    def test_attr_error_reporting_and_fixing(self):
+        n = Node('tag', 'content').with_attr(faulty='<&"')
+        set_config_value('xml_attribute_error_handling', 'fail')
+        try:
+            s = n.as_xml()
+            assert False, "ValueError expected"
+        except ValueError:
+            pass
+        set_config_value('xml_attribute_error_handling', 'fix')
+        assert n.as_xml() == '''<tag faulty='&lt;&amp;"'>content</tag>''', n.as_xml()
+        set_config_value('xml_attribute_error_handling', 'ignore')
+        assert n.as_xml() == '''<tag faulty='<&"'>content</tag>'''
+        n.attr['nonascii'] = 'ἱεραρχικωτάτου'
+        set_config_value('xml_attribute_error_handling', 'lxml')
+        assert n.as_xml() == '''<tag faulty='&lt;&amp;"' nonascii="??????????????">content</tag>'''
+
 
 class TestNode:
     """
@@ -233,8 +307,11 @@ class TestNode:
         tags = [node.tag_name
                 for node in self.unique_tree.select_if(lambda nd: True, include_root=True)]
         assert ''.join(tags) == "abdfg", ''.join(tags)
+        tags = [node.tag_name
+                for node in self.unique_tree.select(ANY_NODE, include_root=True, skip_subtree='f')]
+        assert ''.join(tags) == "abdf", ''.join(tags)
 
-    def test_select_context(self):
+    def test_tree_select_context_if(self):
         tree = parse_sxpr(self.unique_nodes_sexpr)
         contexts = []
         for ctx in tree.select_context_if(lambda ctx: ctx[-1].tag_name >= 'd',
@@ -247,9 +324,33 @@ class TestNode:
             contexts.append(''.join(nd.tag_name for nd in ctx))
         assert contexts == ['af', 'afg', 'ad']
 
+    def test_select_context_with_skipping(self):
+        tree = parse_sxpr('(a (b c) (d e) (f (g h)))')
+        check = []
+        contexts = []
+        def select_f(ctx):
+            nonlocal check
+            check.append(''.join(nd.tag_name for nd in ctx))
+            return True
+        for ctx in tree.select_context(select_f, include_root=True):
+            contexts.append(''.join(nd.tag_name for nd in ctx))
+        assert check == contexts == ['a', 'ab', 'ad', 'af', 'afg']
+        check = []
+        contexts = []
+        for ctx in tree.select_context(select_f, include_root=True, skip_subtree='f'):
+            contexts.append(''.join(nd.tag_name for nd in ctx))
+        assert check == contexts == ['a', 'ab', 'ad', 'af']
+
+    def test_tree_select_context(self):
+        tree = parse_sxpr('(A (B 1) (C (X 1) (Y 1)) (B 2))')
+        l = [str(nd) for nd in tree.select('B')]
+        assert ''.join(l) == '12'
+        l = [str(ctx[-1]) for ctx in tree.select_context('B')]
+        assert ''.join(l) == '12'
+
     def test_select_children(self):
         tree = parse_sxpr('(A (B 1) (C (X 1) (Y 1)) (B 2))')
-        children = list(nd.tag_name for nd in tree.select_children(ALL_NODES))
+        children = list(nd.tag_name for nd in tree.select_children(ANY_NODE))
         assert children == ['B', 'C', 'B']
         B_values = list(nd.content for nd in tree.select_children('B', reverse=True))
         assert B_values == ['2', '1']
@@ -267,7 +368,7 @@ class TestNode:
         assert item_w_value_4 not in tree
         assert tree[item_w_value_2].equals(Node('B', '2'))
         try:
-            tree[item_w_value_4]
+            _ = tree[item_w_value_4]
             assert False
         except KeyError:
             pass
@@ -355,6 +456,70 @@ class TestNode:
         node = Node('tag', '<&>')
         assert node.as_xml() == '<tag>&lt;&amp;&gt;</tag>'
 
+    def test_insert_remove(self):
+        node = parse_sxpr('(R (A "1") (B "2") (C "3"))')
+        B = node.pick('B')
+        node.remove(B)
+        assert node.as_sxpr() == '(R (A "1") (C "3"))'
+        node.insert(0, B)
+        assert node.as_sxpr() == '(R (B "2") (A "1") (C "3"))'
+
+    def test_as_etree(self):
+        import xml.etree.ElementTree as ET
+        # import lxml.etree as ET
+        sxpr = '(R (A "1") (S (B `(class "bold") "2")) (C "3"))'
+        xml = '<R><A>1</A><S><B class="bold">2</B></S><C>3</C></R>'
+        node = parse_sxpr(sxpr)
+        et = node.as_etree()
+        assert ET.tostring(et, encoding="unicode") == xml, ET.tostring(et, encoding="unicode")
+        node = Node.from_etree(et)
+        assert node.as_sxpr() == sxpr
+        et = ET.XML('<R>mixed <A>1</A>mode <!-- comment --><B class="italic" /></R>')
+        node = Node.from_etree(et)
+        expected_sxpr = '(R (:Text "mixed ") (A "1") (:Text "mode ") (B `(class "italic")))'
+        assert node.as_sxpr() == expected_sxpr
+        et = node.as_etree()
+        et = ET.XML(ET.tostring(et, encoding="unicode"))
+        node = Node.from_etree(et)
+        assert node.as_sxpr() == expected_sxpr
+        empty_tags = set()
+        tree = parse_xml('<a><b>1<c>2<d />3</c></b>4</a>', out_empty_tags=empty_tags)
+        etree = tree.as_etree(empty_tags=empty_tags)
+        assert ET.tostring(etree).replace(b' /', b'/') == b'<a><b>1<c>2<d/>3</c></b>4</a>'
+        tree = Node.from_etree(etree)
+        assert flatten_sxpr(tree.as_sxpr()) == \
+               '(a (b (:Text "1") (c (:Text "2") (d) (:Text "3"))) (:Text "4"))'
+
+    def test_delete_item(self):
+        sxpr = '(root (A "0") (B "1") (C "2") (D "3"))'
+        node = parse_sxpr(sxpr)
+        try:
+            _ = node.index('E')
+            assert False, 'ValueError expected'
+        except ValueError:
+            pass
+        assert node[-1].tag_name == "D"
+        try:
+            del node[4]
+            assert False, 'IndexError expected'
+        except IndexError:
+            pass
+        try:
+            del node[-5]
+            assert False, 'IndexError expected'
+        except IndexError:
+            pass
+        del node[-2]
+        assert 'C' not in node
+        del node[0]
+        assert 'A' not in node
+        sxpr = '(root (A "0") (B "1") (C "2") (D "3"))'
+        node = parse_sxpr(sxpr)
+        del node[1:3]
+        assert str(node) == "03"
+
+
+
 
 class TestRootNode:
     def test_error_handling(self):
@@ -380,24 +545,24 @@ class TestRootNode:
         # wrong
         number = RE(r'\d+') | RE(r'\d+') + RE(r'\.') + RE(r'\d+')
         result = str(Grammar(number)("3.1416"))
-        assert result.startswith('3 <<< Error on ".1416" | Parser stopped before end!'), \
+        assert result.startswith('3 <<< Error on ".1416" | Parser "root" stopped before end'), \
             str(result)
 
     def test_get_errors(self):
-        # TODO: extend this test to the more compilcated case of removed nodes
+        # TODO: extend this test to the more complicated case of removed nodes
         number = RE(r'\d+') + RE(r'\.') + RE(r'\d+') | RE(r'\d+')
         result = Grammar(number)("3.1416")
-        assert not result.get_errors(result.children[1])
-        assert not result.get_errors(result)
+        assert not result.node_errors(result.children[1])
+        assert not result.node_errors(result)
 
         result = Grammar(number)("x.1416")
-        assert result.get_errors(result[0])
-        assert not result.get_errors(result)
+        assert result.node_errors(result[0])
+        assert not result.node_errors(result)
 
         number = RE(r'\d+') | RE(r'\d+') + RE(r'\.') + RE(r'\d+')
         result = Grammar(number)("3.1416")
-        assert not result.get_errors(result)
-        assert result.get_errors(result[1])
+        assert not result.node_errors(result)
+        assert result.node_errors(result[1])
 
     def test_copy_errors(self):
         tree = RootNode(parse_sxpr('(A (B "1") (C "2"))').with_pos(0))
@@ -484,23 +649,23 @@ class TestSerialization:
 
     def test_sexpr(self):
         tree = parse_sxpr('(A (B "C") (D "E"))')
-        s = tree.as_sxpr(flatten_threshold=0)
+        s = tree.as_sxpr(compact=False, flatten_threshold=0)
         assert s == '(A\n  (B\n    "C"\n  )\n  (D\n    "E"\n  )\n)', s
         tree = parse_sxpr('(A (B (C "D") (E "F")) (G "H"))')
-        s = tree.as_sxpr(flatten_threshold=0)
+        s = tree.as_sxpr(compact=False, flatten_threshold=0)
         assert s == '(A\n  (B\n    (C\n      "D"\n    )\n    (E\n      "F"\n    )' \
-            '\n  )\n  (G\n    "H"\n  )\n)', s
+            '\n  )\n  (G\n    "H"\n  )\n)'
         tree = parse_sxpr('(A (B (C "D\nX") (E "F")) (G " H \n Y "))')
-        s = tree.as_sxpr(flatten_threshold=0)
+        s = tree.as_sxpr(compact=False, flatten_threshold=0)
         assert s == '(A\n  (B\n    (C\n      "D"\n      "X"\n    )' \
             '\n    (E\n      "F"\n    )\n  )\n  (G\n    " H "\n    " Y "\n  )\n)', s
 
     def test_compact_representation(self):
         tree = parse_sxpr('(A (B (C "D") (E "F")) (G "H"))')
-        compact = tree.as_sxpr(compact=True)
+        compact = tree.as_sxpr(compact=True, flatten_threshold=0)
         assert compact == '(A\n  (B\n    (C "D")\n    (E "F"))\n  (G "H"))'
         tree = parse_sxpr('(A (B (C "D\nX") (E "F")) (G " H \n Y "))')
-        compact = tree.as_sxpr(compact=True)
+        compact = tree.as_sxpr(compact=True, flatten_threshold=0)
         assert compact == '(A\n  (B\n    (C\n      "D"\n      "X")\n    (E "F"))' \
             '\n  (G\n    " H "\n    " Y "))'
         tree = parse_sxpr('(A (B (C "D") (E "F")) (G "H"))')
@@ -509,9 +674,10 @@ class TestSerialization:
         threshold = get_config_value('flatten_sxpr_threshold')
         set_config_value('flatten_sxpr_threshold', 20)
         compact = tree.serialize('indented')
-        assert compact == 'A\n  B\n    C `(attr "val")\n      "D"\n    E\n      "F"\n  G\n    "H"'
+        # assert compact == 'A\n  B\n    C `(attr "val")\n      "D"\n    E\n      "F"\n  G\n    "H"'
+        assert compact == 'A\n  B\n    C `(attr "val") "D"\n    E "F"\n  G "H"', compact
         tree = parse_xml('<note><priority level="high" /><remark></remark></note>')
-        assert tree.serialize(how=INDENTED_SERIALIZATION) == 'note\n  priority `(level "high")\n  remark'
+        assert tree.serialize(how='indented') == 'note\n  priority `(level "high")\n  remark'
         set_config_value('flatten_sxpr_threshold', threshold)
 
     def test_xml_inlining(self):
@@ -519,7 +685,6 @@ class TestSerialization:
 
         xml = tree.as_xml(inline_tags={'A'})
         assert xml == "<A><B>C</B><D>E</D></A>", xml
-
         assert tree.as_xml() == "<A>\n  <B>C</B>\n  <D>E</D>\n</A>", xml
 
         tree.attr['xml:space'] = 'preserve'
@@ -543,10 +708,10 @@ class TestSerialization:
     def test_xml_tag_omission(self):
         tree = parse_sxpr('(XML (T "Hallo") (L " ") (T "Welt!"))')
         all_tags = {'XML', 'T', 'L'}
-        assert tree.as_xml(inline_tags=all_tags, omit_tags=all_tags) == "Hallo Welt!"
+        assert tree.as_xml(inline_tags=all_tags, string_tags=all_tags) == "Hallo Welt!"
         # tags with attributes will never be ommitted
         tree.pick_child('T').attr['class'] = "kursiv"
-        assert tree.as_xml(inline_tags=all_tags, omit_tags=all_tags) == \
+        assert tree.as_xml(inline_tags=all_tags, string_tags=all_tags) == \
                '<T class="kursiv">Hallo</T> Welt!'
 
 
@@ -655,6 +820,52 @@ class TestContextNavigation:
         c = next_context(ctx)
         assert c[-1].tag_name == 'C'
 
+    def test_context_mapping(self):
+        tree = parse_sxpr('(A (B alpha) (C (D beta) (E gamma)) (F delta))')
+        cm = generate_context_mapping(tree)
+        for i in range(len(tree.content)):
+            ctx, rel_pos = map_pos_to_context(i, cm)
+        i = tree.content.find("delta")
+        ctx, rel_pos = map_pos_to_context(i, cm)
+        assert ctx[-1].tag_name == 'F' and rel_pos == 0
+        i = tree.content.find("lta")
+        ctx, rel_pos = map_pos_to_context(i, cm)
+        assert ctx[-1].tag_name == 'F' and rel_pos == 2
+        i = tree.content.find("a")
+        ctx, rel_pos = map_pos_to_context(i, cm)
+        assert ctx[-1].tag_name == 'B' and rel_pos == 0
+        i = tree.content.rfind("a")
+        ctx, rel_pos = map_pos_to_context(i, cm)
+        assert ctx[-1].tag_name == 'F' and rel_pos == 4
+        i = tree.content.find("mm")
+        ctx, rel_pos = map_pos_to_context(i, cm)
+        assert ctx[-1].tag_name == 'E' and rel_pos == 2
+
+    def test_standalone_select_context_if(self):
+        start = self.tree.pick_context('E')
+        save = start.copy()
+        sequence = []
+        for ctx in select_context_if(
+                start, lambda c: True, include_root=True, reverse=True):
+            sequence.append(''.join(n.tag_name for n in ctx))
+        assert sequence == ['ACE', 'ACD', 'AC', 'AB', 'A']
+        assert save == start  # context passed should not be changed by select_context
+
+        start = self.tree.pick_context('D')
+        sequence = []
+        for ctx in select_context_if(
+                start, lambda c: True, include_root=True, reverse=False):
+            sequence.append(''.join(n.tag_name for n in ctx))
+        assert sequence == ['ACD', 'ACE', 'AC', 'AF', 'A']
+
+    def test_standalone_pick_context(self):
+        start = self.tree.pick_context('A', include_root=True)
+        anfang = pick_context(start, LEAF_CONTEXT, include_root=True)
+        ende = pick_context(start, LEAF_CONTEXT, include_root=True, reverse=True)
+        assert anfang[-1].tag_name == 'B'
+        assert ende[-1].tag_name == 'F'
+
+
 if __name__ == "__main__":
     from DHParser.testing import runner
     runner("", globals())
