@@ -20,20 +20,21 @@ limitations under the License.
 """
 
 import collections.abc
+import copy
 import os
 import sys
 
 scriptpath = os.path.dirname(__file__) or '.'
 sys.path.append(os.path.abspath(os.path.join(scriptpath, '..')))
 
-from DHParser.configuration import SXPRESSION_SERIALIZATION
-from DHParser.syntaxtree import Node, parse_sxpr, parse_xml, PLACEHOLDER, \
+from DHParser.syntaxtree import Node, RootNode, parse_sxpr, parse_xml, PLACEHOLDER, \
     tree_sanity_check, flatten_sxpr, WHITESPACE_PTYPE
 from DHParser.transform import traverse, reduce_single_child, remove_whitespace, move_adjacent, \
     traverse_locally, collapse, collapse_children_if, lstrip, rstrip, remove_content, \
     remove_tokens, transformation_factory, has_ancestor, has_parent, contains_only_whitespace, \
-    merge_adjacent, is_one_of, swap_attributes, delimit_children, \
-    positions_of, insert, node_maker, apply_if, change_tag_name, add_attributes
+    merge_adjacent, is_one_of, not_one_of, swap_attributes, delimit_children, merge_treetops, \
+    positions_of, insert, node_maker, apply_if, change_tag_name, add_attributes, \
+    merge_leaves, BLOCK_ANONYMOUS_LEAVES
 from typing import AbstractSet, List, Sequence, Tuple
 
 
@@ -172,9 +173,9 @@ class TestConditionalTransformations:
         context = [Node('C', 'alpha'),
                    Node('B', 'beta'),
                    Node('A', 'gamma')]
-        assert not has_ancestor(context, {'A'})
-        assert has_ancestor(context, {'B'})
-        assert not has_ancestor(context, {'C'})
+        assert not has_ancestor(context, {'A'}, 1)
+        assert has_ancestor(context, {'B'}, 1)
+        assert not has_ancestor(context, {'C'}, 1)
         assert has_ancestor(context, {'C'}, 2)
 
         assert not has_parent(context, {'A'})
@@ -307,6 +308,18 @@ class TestWhitespaceTransformations:
         traverse(sentence, transformations)
         assert sentence.content == "Hallo Welt", sentence.content
 
+    # def test_merge_adjacent2(self):
+    #     expr = parse_sxpr('(Autor (:Whitespace " ") (DEU_GEMISCHT "K. ") (DEU_GEMISCHT "Figala"))')
+    #     transformations = {'Autor': merge_adjacent(not_one_of('Anker'), 'TEXT')}
+    #     traverse(expr, transformations)
+    #     print(expr.as_sxpr())
+
+
+    def test_merge_adjacent_with_attributes(self):
+        tree = parse_sxpr('(A (L " ") (L `(spatium "3+1") " "))')
+        merge_adjacent([tree], is_one_of('L'), 'L')
+        assert len(tree.children) == 1
+        assert tree.children[0].get_attr('spatium', 'FEHLER') == "3+1"
 
 class TestAttributeHandling:
     def test_swap_attributes(self):
@@ -330,7 +343,7 @@ class TestConstructiveTransformations:
         tree = parse_sxpr('(A (B 1) (B 2) (B 3))').with_pos(0)
         trans_table = {'A': delimit_children(node_maker('c', ','))}
         traverse(tree, trans_table)
-        original_result = tree.serialize(how=SXPRESSION_SERIALIZATION)
+        original_result = tree.serialize(how='S-expression')
         assert original_result == '(A (B "1") (c ",") (B "2") (c ",") (B "3"))', original_result
 
     def test_complex_delimiter(self):
@@ -401,6 +414,33 @@ class TestBoolean:
         traverse(tree, trans_table)
         assert flatten_sxpr(tree.as_sxpr()) == '(A (X `(renamed "True") "1") (C "1") (X `(renamed "True") "2"))'
 
+class TestOptimizations:
+    model = RootNode(parse_sxpr('''(array
+          (number "1")
+          (number
+            (:RegExp "2")
+            (:RegExp ".")
+            (:RegExp "0"))
+          (string "a string"))''')).with_pos(0)
+
+    def raise_error(self, context):
+        raise AssertionError()
+
+    def test_squeeze_tree(self):
+        tree = copy.deepcopy(TestOptimizations.model)
+        merge_treetops(tree)
+        assert tree.as_sxpr() == '''(array (number "1") (number "2.0") (string "a string"))'''
+
+    def test_blocking(self):
+        tree = copy.deepcopy(TestOptimizations.model)
+        transtable = {
+            '<': BLOCK_ANONYMOUS_LEAVES,
+            'number': [merge_leaves, reduce_single_child],
+            ':RegExp': self.raise_error
+        }
+        traverse(tree, transtable)
+        assert tree.equals(parse_sxpr('(array (number "1") (number "2.0") (string "a string"))'))
+
 
 if __name__ == "__main__":
     from DHParser.testing import runner
